@@ -44,6 +44,12 @@ get_u64(const void *p)
   return (((u64) ntohl(xh)) << 32) | ntohl(xl);
 }
 
+static inline void
+put_u8(void *p, u8 x)
+{
+  memcpy(p, &x, 1);
+}
+
 static inline void
 put_u16(void *p, u16 x)
 {
