@@ -504,7 +504,8 @@ rip_update_bfd(struct rip_proto *p, struct rip_neighbor *n)
      */
     ip_addr saddr = rip_is_v2(p) ? n->ifa->sk->saddr : n->nbr->ifa->ip;
     n->bfd_req = bfd_request_session(p->p.pool, n->nbr->addr, saddr,
-				     n->nbr->iface, rip_bfd_notify, n);
+				     n->nbr->iface, p->p.vrf,
+				     rip_bfd_notify, n);
   }
 
   if (!use_bfd && n->bfd_req)
