@@ -54,6 +54,7 @@
    this to gen small latencies */
 #define MAX_RX_STEPS 4
 
+
 /*
  *	Tracked Files
  */
@@ -88,19 +89,32 @@ static struct resclass rf_class = {
   NULL
 };
 
-void *
-tracked_fopen(pool *p, char *name, char *mode)
+struct rfile *
+rf_open(pool *p, char *name, char *mode)
 {
   FILE *f = fopen(name, mode);
 
-  if (f)
-    {
-      struct rfile *r = ralloc(p, &rf_class);
-      r->f = f;
-    }
-  return f;
+  if (!f)
+    return NULL;
+
+  struct rfile *r = ralloc(p, &rf_class);
+  r->f = f;
+  return r;
 }
 
+void *
+rf_file(struct rfile *f)
+{
+  return f->f;
+}
+
+int
+rf_fileno(struct rfile *f)
+{
+  return fileno(f->f);
+}
+
+
 /**
  * DOC: Timers
  *
@@ -478,6 +492,20 @@ tm_format_datetime(char *x, struct timeformat *fmt_spec, bird_clock_t t)
     strcpy(x, "<too-long>");
 }
 
+int
+tm_format_real_time(char *x, size_t max, const char *fmt, bird_clock_t t)
+{
+  struct tm tm;
+
+  if (!localtime_r(&t, &tm))
+    return 0;
+
+  if (!strftime(x, max, fmt, &tm))
+    return 0;
+
+  return 1;
+}
+
 
 /**
  * DOC: Sockets
@@ -1574,6 +1602,7 @@ sk_sendmsg(sock *s)
   struct iovec iov = {s->tbuf, s->tpos - s->tbuf};
   byte cmsg_buf[CMSG_TX_SPACE];
   sockaddr dst;
+  int flags = 0;
 
   sockaddr_fill(&dst, s->af, s->daddr, s->iface, s->dport);
 
@@ -1584,6 +1613,13 @@ sk_sendmsg(sock *s)
     .msg_iovlen = 1
   };
 
+#ifdef CONFIG_DONTROUTE_UNICAST
+  /* FreeBSD silently changes TTL to 1 when MSG_DONTROUTE is used, therefore we
+     cannot use it for other cases (e.g. when TTL security is used). */
+  if (ipa_is_ip4(s->daddr) && ip4_is_unicast(ipa_to_ip4(s->daddr)) && (s->ttl == 1))
+    flags = MSG_DONTROUTE;
+#endif
+
 #ifdef CONFIG_USE_HDRINCL
   byte hdr[20];
   struct iovec iov2[2] = { {hdr, 20}, iov };
@@ -1599,7 +1635,7 @@ sk_sendmsg(sock *s)
   if (s->flags & SKF_PKTINFO)
     sk_prepare_cmsgs(s, &msg, cmsg_buf, sizeof(cmsg_buf));
 
-  return sendmsg(s->fd, &msg, 0);
+  return sendmsg(s->fd, &msg, flags);
 }
 
 static inline int
